/*
 * Provide access to keypad events on the configured device.
 *
 * Copyright (C) 2018, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/devices/dataspace.h>
#include <l4/devices/keypad-loader.h>

#include "input-event-server.h"
#include "input-keypad-client.h"

#include <l4/re/event>

#include <ipc/cap_alloc.h>
#include <ipc/server.h>

#include "input_server.h"



static void handler(Input_event event, void *priv)
{
  Input_event_server *server = (Input_event_server *) priv;

  server->send_event(event.type, event.code, event.value);
}



int main(void)
{
  /* Load the keypad details from the configured library. */

  Keypad_generic *keypad = load_keypad();
  Input_keypad_client client(keypad);
  void *buffer;

  /* Memory allocation capability for the event data. */

  l4re_ds_t mem = allocate_data(L4_PAGESIZE, &buffer);

  if (l4_is_invalid_cap(mem))
    return 1;

  /* Obtain a capability for the interrupt. */

  l4_cap_idx_t irq = ipc_cap_alloc();

  if (l4_is_invalid_cap(irq))
    return 1;

  /* Event buffer for the data. */

  L4Re::Event_buffer events(buffer, L4_PAGESIZE);

  /* Initialise and register a server object. */

  Input_event_server obj(mem, events);

  /* Attach the event handler and wait for events. */

  client.attach(handler, (void *) &obj);

  /* Bind and start the IPC server loop. */

  if (ipc_server_loop_for(Input, &obj, "ev"))
    return 1;

  return 0;
}
